#!/usr/bin/python
# -*- coding: utf-8 -*-
# ----------------------------------------------------------------------
# Copyright (c) 2012 Michael Hull.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------

from django.core.management import setup_environ
import mreorg.curator.settings as settings
setup_environ(settings)
import django

import time
import os
import datetime
import subprocess
import sys
import signal
import random

from mreorg.curator.frontend.models import SimQueueEntry
from mreorg.curator.frontend.models import SimQueueEntryState
import mreorg


def simulate(sim_queue_entry):
    filename = sim_queue_entry.simfile.full_filename
    print ' - Simulating: ', filename
    (dname, fname) = os.path.split(filename)

    # Update the database to reflect
    print '   - Updating database'
    sim_queue_entry.status = SimQueueEntryState.Executing
    sim_queue_entry.simulation_start_time = datetime.datetime.now()
    sim_queue_entry.simulation_last_heartbeat = datetime.datetime.now()
    sim_queue_entry.save(force_update=True)

    def try_heartbeat():
        try:
            sim_queue_entry.simulation_last_heartbeat = datetime.datetime.now()
            sim_queue_entry.save(force_update=True)
        except django.db.utils.DatabaseError:
            pass

    # First heartbeat:
    try_heartbeat()

    # Setup the environmental variables:
    # Pass the RunConfiguration.id as an environmental variable

    os.environ['_MREORG_RUNCONFIGID'] = str(sim_queue_entry.runconfig.id)

    if sim_queue_entry.runconfig.timeout:
        os.environ['MREORG_CONFIG'] =os.environ.get('MREORG_CONFIG') + ';TIMEOUT=%d' % sim_queue_entry.runconfig.timeout

    for envvar in sim_queue_entry.runconfig.environvar_set.all():
        key = envvar.key
        value = envvar.value
        if value is None:
            if key in os.environ:
                del os.environ[key]
        else:
            os.environ[key] = value

    # Setup the heartbeat, to say that we are actually alive:
    heartbeat_interval = mreorg.MReOrgConfig.config['Settings']['Curate']['backend_heartbeat_rate']

    def handler(*args, **kwargs):
        signal.alarm(heartbeat_interval)
        try_heartbeat()

    signal.alarm(heartbeat_interval)
    signal.signal(signal.SIGALRM, handler)

    # Simulate:
    print '   - Changing Directory to', dname
    os.chdir(dname)
    try:
        subprocess.check_call(['python', fname])
        print '   - Finished Simulating [Exit OK]'
    except subprocess.CalledProcessError, exception:
        print '   - Finished Simulating [Non-zero exitcode]'
        last_run = sim_queue_entry.simfile.get_last_run(sim_queue_entry.runconfig)
        if not last_run:
            print 'Sim not decorated! Unable to set return code'
        else:
            last_run.returncode = exception.returncode
            last_run.save(force_update=True)

    # Turn off heartbeating.
    signal.alarm(0)
    try_heartbeat()

    # Remove the sim_queue_entry:
    sim_queue_entry.delete()
    sim_queue_entry.simfile.recache_from_filesystem()


def _run_backend():

    os.environ['MREORG_CONFIG']=os.environ.get('MREORG_CONFIG','') + ';CURATIONRUN'
    os.environ['MREORG_CONFIG']=os.environ.get('MREORG_CONFIG','') + ';ENABLECOVERAGE'


    while True:

        print '\r Checking for Queued Sims: ', time.strftime('%l:%M%p (%S) on %b %d, %Y'),
        sys.stdout.flush()
        queued_objects = SimQueueEntry.objects.\
                filter(status=SimQueueEntryState.Waiting).\
                order_by('submit_time')

        if queued_objects:
            print
            simulate(queued_objects[0])
            print

        time.sleep(random.randint(3,20))

def run_backend():
    try:
        _run_backend()
    except KeyboardInterrupt:
        print 'Stopping'


if __name__ == '__main__':
    run_backend()




#!/usr/bin/python
# -*- coding: utf-8 -*-

from pyfile_astwalker import MyFileWalker
import re


class PyFile(object):

    def __init__(self, filename):
        self.filename = filename
        assert filename.endswith('.py')

        with open(self.filename) as f:
            self.contents = f.read()

    def get_cleaned_version(self):
        txt = self.contents
        toggle_block = '##==TOGGLE=='
        if toggle_block in txt:
            blocks = txt.split(toggle_block)
            txt = ''.join(blocks[1::2])

        txt = txt.strip()
        return txt


class PyFileScons(PyFile):
    def __init__(self, filename):
        super(PyFileScons,self).__init__(filename=filename)
        
        
        
    def get_output_files(self):
        includes_1 = MyFileWalker.get_all_func_calls(self.contents, 'savefig', argpos=0)
        includes_2 = MyFileWalker.get_all_func_calls(self.contents, 'save', argpos=0)

        # OUTPUT: 'filename' lines
        includes_3 = []
        re_outputline = re.compile(r"""#\s*OUTPUT:\s*'(.*)'\s*$""", re.MULTILINE)
        for match in re_outputline.finditer(self.contents):
            includes_3.append(match.groups()[0])

        includes = includes_1 + includes_2 + includes_3

        return includes

    def get_dependancy_files(self):

        includes1a = MyFileWalker.get_all_func_calls(self.contents, 'add_file', argpos=1)
        includes1b = MyFileWalker.get_all_func_calls(self.contents, 'add_file_mpl', argpos=1)
        includes = includes1a + includes1b

        includes2 = []
        re_outputline = re.compile(r"""#\s*INPUT:\s*'(.*)'\s*$""", re.MULTILINE)
        for match in re_outputline.finditer(self.contents):
            includes2.append(match.groups()[0])

        return includes + includes2




"""
xmlwitch offers Pythonic XML generation through context generators in a 
minimalist implementation with less than 100 lines of code. BSD-licensed.

Usage
`````

::

    import xmlwitch
    xml = xmlwitch.Builder(version='1.0', encoding='utf-8')
    with xml.feed(xmlns='http://www.w3.org/2005/Atom'):
        xml.title('Example Feed')
        xml.updated('2003-12-13T18:30:02Z')
        with xml.author:
            xml.name('John Doe')
        xml.id('urn:uuid:60a76c80-d399-11d9-b93C-0003939e0af6')
        with xml.entry:
            xml.title('Atom-Powered Robots Run Amok')
            xml.id('urn:uuid:1225c695-cfb8-4ebb-aaaa-80da344efa6a')
            xml.updated('2003-12-13T18:30:02Z')
            xml.summary('Some text.')
    print(xml)

Setup
`````

::

    $ pip install xmlwitch # or
    $ easy_install xmlwitch # or
    $ cd xmlwitch-0.2; python setup.py install

Links
`````

* `Full documentation <http://jonasgalvez.com.br/Software/XMLWitch.html>`_
* `Development repository <http://github.com/galvez/xmlwitch/>`_
* `Author's website <http://jonasgalvez.com.br/>`_

"""

from distutils.core import setup

setup(
    name = 'xmlwitch',
    version = '0.2.1',
    url = 'http://jonasgalvez.com.br/Software/XMLWitch.html',
    license = 'BSD',
    author = "Jonas Galvez",
    author_email = "jonasgalvez@gmail.com",
    description = "xmlwitch offers Pythonic XML generation through context generators",
    long_description = __doc__,
    py_modules = ['xmlwitch'],
    platforms = 'Python 2.5 and later',
    classifiers = [
        'Intended Audience :: Developers',
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries',
        'Topic :: Text Processing :: Markup :: HTML',
        'Topic :: Text Processing :: Markup :: XML'
    ]
)


#!/usr/bin/python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# Copyright (c) 2012 Michael Hull.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------
import os
import datetime
import StringIO

from django.db import models
from pygments import highlight
from pygments.lexers import PythonLexer
from pygments.formatters import HtmlFormatter

import mreorg


class Options(object):

    MinimumFileCheckInterval = datetime.timedelta(minutes=5)


class SourceSimDir(models.Model):

    class Meta:

        ordering = ['directory_name']

    directory_name = models.CharField(max_length=1000)
    should_recurse = models.BooleanField()

    def does_exist(self):
        return os.path.exists(self.directory_name)

    @classmethod
    def create(cls, directory_name, should_recurse=True):
        directory_name = os.path.normpath(directory_name)
        if SourceSimDir.objects.filter(directory_name=directory_name).count() !=0:
            return

        # Create and save
        p = SourceSimDir(directory_name=directory_name, should_recurse=should_recurse)
        p.save()


class TrackingStatus(object):

    Tracked = 'Tracked'
    NotTracked = 'Nottracked'


class SimFile(models.Model):

    class Meta:
        ordering = ['full_filename']

    @classmethod
    def get_or_make(self, full_filename, make_kwargs=None):
        full_filename = os.path.expanduser(full_filename)

        try:
            simfile = SimFile.objects.get(full_filename=full_filename)
        except SimFile.DoesNotExist:
            make_kwargs = make_kwargs or {}
            simfile = SimFile(full_filename=full_filename, **make_kwargs)
            simfile.save()
        return simfile




    full_filename = models.CharField(max_length=1000)
    tracking_status = models.CharField(
            max_length=1000,
            default=TrackingStatus.NotTracked,
            choices= [
                (TrackingStatus.Tracked,TrackingStatus.Tracked),
                (TrackingStatus.NotTracked,TrackingStatus.NotTracked),
                ] )

    # We cache the docstring, the code, code rendered as HTML:
    last_read_sha1 = models.CharField(max_length=100000, null=True)
    last_read_time = models.DateTimeField(null=True)
    last_read_contents = models.CharField(max_length=100000, null=True)
    last_read_docstring = models.CharField(max_length=100000, null=True)
    last_read_htmlcode = models.CharField(max_length=100000, null=True)

    # Last simulation run:
    #last_run = models.ForeignKey('SimFileRun',null=True, related_name='+')

    @classmethod
    def get_tracked_sims(cls, **kwargs):
        if kwargs:
            return SimFile.objects.get(tracking_status=TrackingStatus.Tracked, **kwargs)
        return SimFile.objects.filter(tracking_status=TrackingStatus.Tracked)

    @classmethod
    def get_untracked_sims(cls,**kwargs):
        if kwargs:
            return SimFile.objects.get(tracking_status=TrackingStatus.NotTracked, **kwargs)
        return SimFile.objects.filter(tracking_status=TrackingStatus.NotTracked)


    # Handle Caching:
    # ##########################################
    def _is_cache_valid(self):
        if not self.last_read_time:
            return False
        # We only maximally recalculate the hash every so often:
        if self.last_read_time - datetime.datetime.now() < Options.MinimumFileCheckInterval:
            return True
        if self.get_current_checksum() == self.last_read_sha1:
            return True
        return False

    def recache_from_filesystem(self):
        print 'Recaching: ', self.full_filename
        with open(self.full_filename) as f:
            code = f.read()

        self.last_read_sha1 = self.get_current_checksum()
        self.last_read_time = datetime.datetime.now()
        self.last_read_contents  = code
        self.last_read_docstring = mreorg.utils.extract_docstring_from_fileobj(StringIO.StringIO( code ) )
        self.last_read_htmlcode = highlight(code, PythonLexer(), HtmlFormatter())
        self.save()


    def check_and_recache(self):
        if not self._is_cache_valid():
            self.recache_from_filesystem()
    # ##########################################



    def does_file_exist(self):
        return os.path.exists(self.full_filename)


    def get_html_code(self):
        self.check_and_recache()
        return self.last_read_htmlcode

    def get_docstring(self):
        self.check_and_recache()
        return self.last_read_docstring

    def _get_last_run(self, runconfig):
        runs = self.get_runs(runconfig=runconfig)
        if len(runs) == 0:
            return None
        else:
            return self.get_runs(runconfig=runconfig)[0]

    def get_last_run(self, runconfig):
        if runconfig.id not in self.latest_run_dict:
            self.latest_run_dict[runconfig.id] = self._get_last_run(runconfig)
        return self.latest_run_dict[runconfig.id]


    def get_current_checksum(self):
        return mreorg.get_file_sha1hash(self.full_filename)

    def get_short_filename(self):
        return os.path.split(self.full_filename)[1]

    def get_runs(self, runconfig):
        return SimFileRun.objects. \
                    filter(simfile=self.id, runconfig=runconfig). \
                    order_by('-execution_date')


    def get_status(self, runconfig):
        last_run = self.get_last_run(runconfig)
        if not last_run:
            return SimRunStatus.NeverBeenRun
        else:
            return last_run.get_status()

    def get_last_executiontime(self, runconfig):
        last_run = self.get_last_run(runconfig)
        if not last_run:
            return 'Unknown'
        else:
            return last_run.execution_time

    def is_queued(self, runconfig):
        return self.simqueueentry_set.filter(runconfig=runconfig).count() != 0

    def getCSSQueueState(self, runconfig):
        p = self.is_queued(runconfig=runconfig)
        LUT = {True: 'SimQueued', False: 'SimNotQueued'}
        return LUT[p]


    def is_currently_running(self,runconfig):
        #print 'Lying about whether I am currently running'
        return False



def extra_init_for_simfile(instance, **kwargs):
    #print 'extra init for simfile', kwargs
    instance.latest_run_dict = {}


from django.db.models.signals import post_init
post_init.connect(extra_init_for_simfile, SimFile)     



class RunConfiguration(models.Model):

    special_configs = ['default']

    @classmethod
    def get_or_make(self, name, make_kwargs=None):
        try:
            runconf = RunConfiguration.objects.get(name=name)
        except RunConfiguration.DoesNotExist:
            if make_kwargs is None:
                make_kwargs = {}
            runconf = RunConfiguration(name=name, **make_kwargs)
            runconf.save()
        return runconf

    class Meta:

        ordering = ('name', )

    name = models.CharField(max_length=10000000)
    timeout = models.IntegerField(null=True)

    def is_special(self):
        return self.name in RunConfiguration.special_configs

    @classmethod
    def build_all_specials(cls):
        for special_name in RunConfiguration.special_configs:
            rc = cls.get_or_make(special_name)
            assert rc.is_special()
            rc.save()

    @classmethod
    def get_initial(cls):
        return cls.get_or_make(name='default')


class FileGroup(models.Model):

    special_groups = ['all']

    class Meta:

        ordering = ('name', )

    name = models.CharField(max_length=10000000)
    simfiles = models.ManyToManyField(SimFile)

    @classmethod
    def get_or_make(self, name):
        try:
            fg = FileGroup.objects.get(name=name)
        except FileGroup.DoesNotExist:
            fg = FileGroup(name=name)
            fg.save()
        return fg

    def is_special(self):
        return self.name in FileGroup.special_groups

    def display_name(self):
        if self.is_special():
            return '--%s--' % self.name
        else:
            return self.name

    def contains_simfile(self, simfile):
        print 'Does', self.name, 'contain', simfile.full_filename, '?'
        if self.name == 'all':
            return True

        assert not self.is_special(), 'NotImplementedYet'
        res = self.simfiles.filter(id=simfile.id).count() > 0
        return res

    @classmethod
    def build_all_specials(cls):
        for special_name in FileGroup.special_groups:
            rc = cls.get_or_make(special_name)
            assert rc.is_special()
            rc.save()

    @classmethod
    def get_initial(cls):
        return cls.get_or_make(name='all')

    @property
    def tracked_sims(self):
        return self.tracked_files

    @property
    def tracked_files(self):
        if self.is_special():
            if self.name == 'all':
                srclist = SimFile.objects.all()
            else:
                assert False, ''
        else:
            srclist = self.simfiles

        return srclist.filter(tracking_status=TrackingStatus.Tracked)


class EnvironVar(models.Model):

    key = models.CharField(max_length=10000)
    value = models.CharField(max_length=10000, null=True)
    config = models.ForeignKey(RunConfiguration)


class SimRunStatus(object):

    Success = 'Success'
    UnhandledException = 'UnhandledException'
    TimeOut = 'Timeout'
    NonZeroExitCode = 'NonZeroExitCode'
    FileChanged = 'FileChanged'
    NeverBeenRun = 'NeverBeenRun'


class SimFileRun(models.Model):

    simfile = models.ForeignKey(SimFile)
    runconfig = models.ForeignKey(RunConfiguration)
    execution_date = models.DateTimeField('execution date')
    return_code = models.IntegerField()
    std_out = models.CharField(max_length=10000000)
    std_err = models.CharField(max_length=10000000)
    exception_type = models.CharField(max_length=10000,null=True, blank=False)
    exception_traceback = models.CharField(max_length=10000,null=True, blank=False)

    simulation_sha1hash = models.CharField(max_length=200)
    library_sha1hash = models.CharField(max_length=200)
    execution_time = models.IntegerField(null=True)

    def execution_data_string(self):
        import datetime
        exec_date = str(self.execution_date)
        d = datetime.datetime.strptime(exec_date,'%Y-%m-%d %H:%M:%S.%f')
        s = d.strftime('%Y-%m-%d %H:%M')
        return s

    def is_script_uptodate(self):
        res = str(self.simulation_sha1hash) == str(self.simfile.get_current_checksum())
        return res

    def get_status(self):
        if self.simulation_sha1hash != self.simfile.get_current_checksum():
            return SimRunStatus.FileChanged
        if self.execution_time is None:
            return SimRunStatus.TimeOut
        if self.exception_type:
            if 'TimeoutException' in self.exception_type:
                return SimRunStatus.TimeOut

            return SimRunStatus.UnhandledException
        if self.return_code != 0:
            return SimRunStatus.NonZeroExitCode
        return SimRunStatus.Success


class SimFileRunOutputImage(models.Model):

    original_name = models.CharField(max_length=10000)
    hash_name = models.CharField(max_length=10000)
    hash_thumbnailname = models.CharField(max_length=10000)
    simulation = models.ForeignKey(SimFileRun, related_name='output_images')

    def hash_name_short(self):
        return self.hash_name.split('/')[-1]

    def hash_thumbnailname_short(self):
        return self.hash_thumbnailname.split('/')[-1]



class SimQueueEntryState(models.Model):

    Waiting = 'Waiting'
    Executing = 'Executing'


class SimQueueEntry(models.Model):

    simfile = models.ForeignKey(SimFile)
    runconfig = models.ForeignKey(RunConfiguration)
    submit_time = models.DateTimeField('submission_time', default=datetime.datetime.now)
    simulation_start_time = models.DateTimeField(null=True, default=None)
    simulation_last_heartbeat = models.DateTimeField(null=True, default=None)
    status = models.CharField(max_length=1000, default=SimQueueEntryState.Waiting )


    @classmethod
    def create(cls, sim_file, runconfig):
        queue_entry = SimQueueEntry(simfile=sim_file,
                                    runconfig=runconfig)
        queue_entry.save()

    def get_simulation_time(self):
        return datetime.datetime.now() - self.simulation_start_time

    def time_since_last_heartbeat_in_s(self):
        if not self.simulation_last_heartbeat:
            return -1
        return (datetime.datetime.now()
                - self.simulation_last_heartbeat).total_seconds()

    def resubmit_if_process_died(self):
        if self.status == SimQueueEntryState.Executing:
            if self.time_since_last_heartbeat_in_s() > 60:
                print 'Resubmitting'
                self.status = SimQueueEntryState.Waiting
                self.save()

    @classmethod
    def trim_dangling_jobs(cls):
        for queue_entry in SimQueueEntry.objects.all():
            queue_entry.resubmit_if_process_died()





#!/usr/bin/python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# Copyright (c) 2012 Michael Hull.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------




#!/usr/bin/python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# Copyright (c) 2012 Michael Hull.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------

import os

from django.conf import settings
from django.conf.urls.defaults import include, patterns
from django.contrib.staticfiles.urls import staticfiles_urlpatterns

from dajaxice.core import dajaxice_autodiscover
dajaxice_autodiscover()

views_mod = 'mreorg.curator.frontend.views'
url_patterns = (
    (r'^$', 'mreorg.curator.frontend.views.view_overview'),


    # Main pages:
    (r'^overview$', views_mod + '.view_overview'),
    (r'^tracking$', views_mod + '.view_tracking'),
    (r'^viewsimulationoutputsummaries', views_mod + '.view_sim_output_summaries'),
    (r'^viewsimulationfailures$', views_mod+ '.view_simulation_failures'),
    (r'^viewsimulationqueue$', views_mod + '.viewsimulationqueue'),
    (r'^viewconfigurations$', views_mod + '.view_configurations'),

    # Details about specific files and runs:
    (r'^simfiles/(\d+)$', views_mod + '.simfile_details'),
    (r'^simfileruns/(\d+)', views_mod + '.simfilerun_details'),
    (r'^simfile/(\d+)', views_mod + '.simfile_details'),

    # Tracking
    (r'^do/track/rescan', views_mod + '.do_track_rescanfs'),
    (r'^do/track/track_sim', views_mod + '.do_track_sim'),
    (r'^do/track/untrack_sim', views_mod + '.do_untrack_sim'),
    (r'^do/track/untrack_src_dir/(\d+)', views_mod + '.do_untrack_src_dir'),
    (r'^do/track/add_src_dir', views_mod + '.do_track_src_dir'),
    (r'^do/track/track_all_sims', views_mod + '.do_track_all'),
    (r'^do/track/untrack_all_sims', views_mod + '.do_untrack_all'),

    # Queuing:
    (r'^do/queue/add_sims', views_mod + '.do_queue_add_sims'),
    (r'^do/editsimfile/(\d+)', views_mod + '.doeditsimfile'),


    # Handle image request
    (r'^image/([\w.]*)$', 'mreorg.curator.frontend.views.get_image_file'),

    # Handle Ajax requests:
    (r'^%s/' % settings.DAJAXICE_MEDIA_PREFIX, include('dajaxice.urls')),
)


urlpatterns = patterns('', *url_patterns) + staticfiles_urlpatterns()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# Copyright (c) 2012 Michael Hull.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------

import os
import re
import inspect, os


class ScriptFlags(object):

    """Control the behaviour of matplotlib within scripts using environmental
    variables. This allows the script to act differently if its being used
    for interactive work or for producing the figures as part of a batch run.

    The environmental variables are read once during intiatisation. The
    behaviours can be changed during the script run by setting the relevant
    class attributes.
    """

    # Allow mreorg.curate to run without needing to set flags:
    entry_point = inspect.stack()[-1][1]
    if 'mreorg.curate' in entry_point and not 'MREORG_CONFIG' in os.environ:
         os.environ['MREORG_CONFIG'] = ''


    if not 'MREORG_CONFIG' in os.environ and not 'mreorg.curate' in entry_point:
        raise RuntimeError(r'''
The environmental variable "MREORG_CONFIG" is not set
Perhaps you should set it, for example:
export MREORG_CONFIG='' # For no changes to Pylab behaviour
export MREORG_CONFIG='SAVEALL;NOSHOW' # To suppress 'show' and save figures instead
        ''')


    _expected_options_new = (
        'NOSHOW',
        'SAVEALL',
        'SAVEFIGADDINFO',
        'CURATIONRUN',
        'BATCHRUN',
        'ENABLECOVERAGE',
        'CURATION_REENTRYFLAG',
        'TIMEOUT',
        'MPLCONFIG',
        'NOMPLIMPORT',

    )

    # OK, lets look for a class named 'MReorgDefaults' in the calling function:
    import inspect
    top_caller = inspect.stack()[-1][0]
    mreorg_defaults = {}
    if 'MReorgDefaults' in top_caller.f_locals:
        for (k,v) in top_caller.f_locals['MReorgDefaults'].__dict__.items():
            if k in ('__doc__','__module__'):
                continue
            assert k in _expected_options_new
            mreorg_defaults[k] = v
            #print 'Using Default:', k, v


    

    # Vreak up the string into a dictionary, for things with an '='
    mreorg_conf_string = os.environ['MREORG_CONFIG']
    mreorg_conf = re.split(r'[,;]', mreorg_conf_string)
    mreorg_conf = [m.strip() for m in mreorg_conf if m.strip()]
    
    mreorg_conf = dict( [m.split("=") if '=' in m else (m,None) for m in mreorg_conf] )
    #print 'ConfigOoptins', mreorg_conf

    for opt in mreorg_conf:
        assert opt in _expected_options_new, 'Unexpected option: %s' % opt


    ENVVAR_MREORG_NOSHOW = ('NOSHOW' in mreorg_conf) or ('NOSHOW' in mreorg_defaults)
    ENVVAR_MREORG_NOMPLIMPORT = ('NOMPLIMPORT' in mreorg_conf) or ('NOMPLIMPORT' in mreorg_conf)
    ENVVAR_MREORG_SAVEALL = ('SAVEALL' in mreorg_conf) or ('SAVEALL' in mreorg_conf)
    ENVVAR_MREORG_SAVEFIGADDINFO = ('SAVEFIGADDINFO' in mreorg_conf) or ('SAVEFIGADDINFO' in mreorg_conf)
    ENVVAR_MREORG_CURATIONRUN = ('CURATIONRUN' in mreorg_conf) or ('CURATIONRUN' in mreorg_conf)
    ENVVAR_MREORG_BATCHRUN = ('BATCHRUN' in mreorg_conf) or ('BATCHRUN' in mreorg_conf)
    ENVVAR_MREORG_ENABLECOVERAGE = ('ENABLECOVERAGE' in mreorg_conf) or ('ENABLECOVERAGE' in mreorg_conf)
    ENVVAR_MREORG_CURATION_REENTRY = ('CURATION_REENTRYFLAG' in mreorg_conf) or ('CURATION_REENTRYFLAG' in mreorg_conf)
    


    # Don't call pylab.show() if ...
    MREORG_NOSHOW = ENVVAR_MREORG_NOSHOW or \
                    ENVVAR_MREORG_CURATIONRUN or \
                    ENVVAR_MREORG_BATCHRUN

    # Save all figures produced in the simulation:
    # This will happen when figures are produced and 'pylab.show()'
    # is called, or when the script ends.
    MREORG_SAVEALL = ENVVAR_MREORG_SAVEALL or \
                     ENVVAR_MREORG_CURATIONRUN or \
                     ENVVAR_MREORG_BATCHRUN

    # Copy, for the sake of interface consistency:
    MREORG_CURATIONRUN = ENVVAR_MREORG_CURATIONRUN

    MREORG_CURATION_REENTRY = ENVVAR_MREORG_CURATION_REENTRY

    # If we are building on read-the-docs, we can't import matplotlib:
    MREORG_DONTIMPORTMATPLOTLIB = 'READTHEDOCS' in os.environ or ENVVAR_MREORG_NOMPLIMPORT

    # Default, lets automatically create directories when they don't exist:
    MREORG_AUTOMAKEDIRS = True

    # Should we enable coverage:
    MREORG_ENABLECOVERAGE = ENVVAR_MREORG_ENABLECOVERAGE

    MREORG_SAVEFIGADDINFO = ENVVAR_MREORG_SAVEFIGADDINFO

    # Setup the environment:
    MREORG_MPLCONFIG = None
    if 'MPLCONFIG' in mreorg_conf:
        MREORG_MPLCONFIG=mreorg_conf['MPLCONFIG']
    elif 'MPLCONFIG' in mreorg_defaults:
        MREORG_MPLCONFIG=mreorg_defaults['MPLCONFIG']

    MREORG_MPLCONFIG_FILE = None
    if MREORG_MPLCONFIG:
        currpath = os.path.dirname(os.path.abspath(__file__))
        mplconfigdir = os.path.join(currpath, '../../mplconfigs/')
        target_config_file = os.path.join(mplconfigdir,MREORG_MPLCONFIG + '.conf')
        if not os.path.exists(target_config_file):
            assert False, "Can't find file: %s" % target_config_file
        MREORG_MPLCONFIG_FILE = target_config_file
        print 'Using config file: ', target_config_file



# Look out for unexpected flags:
for key in os.environ:
    if key.startswith('MREORG') and key != 'MREORG_CONFIG':
        if '=' in key:
            continue
        assert key in ScriptFlags._expected_options_new, 'MREORG config option not recognised: %s. Possible options: [%s]' % (key, ','.join(ScriptFlags._expected_options_new))



#!/usr/bin/python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# Copyright (c) 2012 Michael Hull.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------

from django.utils import simplejson
from dajaxice.decorators import dajaxice_register
from mreorg.curator.frontend.models import SimFile
from mreorg.curator.frontend.models import SimQueueEntry
from mreorg.curator.frontend.models import SimRunStatus
from mreorg.curator.frontend.models import SimQueueEntryState
from mreorg.curator.frontend.models import RunConfiguration
from mreorg.curator.frontend.models import FileGroup
from django.db import transaction


def ensure_config():
    pass


@dajaxice_register
def base_set_runconfig(request, runconfig_id):
    runconfig = RunConfiguration.objects.get(id=int(runconfig_id))
    request.session['current_runconfig'] = runconfig
    return simplejson.dumps({})


@dajaxice_register
def base_set_filegroup(request, filegroup_id):
    filegroup = FileGroup.objects.get(id=int(filegroup_id))
    request.session['current_filegroup'] = filegroup
    return simplejson.dumps({})


@dajaxice_register
def overview_resubmit_simfile(_request, simfile_id):
    ensure_config()
    try:
        sim_file = SimFile.get_tracked_sims(id=simfile_id)
    except:
        return simplejson.dumps({})

    # Existing simulation object
    if not sim_file.simqueueentry_set.count():
        SimQueueEntry.create(
                sim_file=sim_file,
                runconfig=_request.session['current_runconfig'])
    return simplejson.dumps({})


@dajaxice_register
def overview_resubmit_simfile_if_failure(_request, simfile_id):
    try:
        sim_file = SimFile.get_tracked_sims(id=simfile_id)
    except:
        return simplejson.dumps({})

    if sim_file.get_status(runconfig=_request.session['current_runconfig']) == SimRunStatus.Success:
        return simplejson.dumps({})

    if not sim_file.simqueueentry_set.count():
        SimQueueEntry.create(sim_file=sim_file, runconfig=_request.session['current_runconfig'])

    return simplejson.dumps({})


@dajaxice_register
def overview_toggle_simfile_for_resubmit(_request, simfile_id):
    ensure_config()

    try:
        sim_file = SimFile.get_tracked_sims(id=simfile_id)
    except:
        return simplejson.dumps({})

    # Existing simulation object
    if sim_file.simqueueentry_set.count():
        sim_file.simqueueentry_set.all().delete()
    else:
        SimQueueEntry.create(sim_file=sim_file, runconfig=_request.session['current_runconfig'])

    return simplejson.dumps({})


@dajaxice_register
def refreshsimlist(_request):
    assert False, 'Is this used?'
    ensure_config()
    runconfig = _request.session['current_runconfig']
    states = {}
    for simfile in SimFile.get_tracked_sims():
        states[simfile.id] = simfile.get_status()
    return simplejson.dumps({'sim_file_states': states})





@dajaxice_register
def update_queue(_request, action):
    print 'Action', action
    tracked_sim_files = SimFile.get_tracked_sims()
    print 'Tracked sim files:', len(tracked_sim_files)

    runconfig=_request.session['current_runconfig']

    if action == 'add-all':
        with transaction.commit_on_success():
            for simfile in tracked_sim_files:
                if simfile.simqueueentry_set.count() > 0:
                    continue
                SimQueueEntry.create( sim_file=simfile, runconfig=runconfig)
        return simplejson.dumps({})

    if action == 'clear-all':
        SimQueueEntry.objects.all() .filter(status = SimQueueEntryState.Waiting).delete()
        return simplejson.dumps({})

    # TODO! Move functionality into here!
    if action == "add-all-failures":
        with transaction.commit_on_success():
            for simfile in tracked_sim_files:
                if simfile.simqueueentry_set.count() > 0:
                    continue
                if simfile.get_status(runconfig=runconfig) not in (SimRunStatus.TimeOut, SimRunStatus.UnhandledException, SimRunStatus.NonZeroExitCode, SimRunStatus.FileChanged,SimRunStatus.NeverBeenRun):
                    continue
                SimQueueEntry.create( sim_file=simfile, runconfig=_request.session['current_runconfig'])
        return simplejson.dumps({})

    if action == "add-all-failures-not-timeout":
        with transaction.commit_on_success():
            for simfile in tracked_sim_files:
                if simfile.simqueueentry_set.count() > 0:
                    continue
                if simfile.get_status(runconfig=runconfig) not in (SimRunStatus.UnhandledException, SimRunStatus.NonZeroExitCode, SimRunStatus.FileChanged,SimRunStatus.NeverBeenRun):
                    continue
                SimQueueEntry.create( sim_file=simfile, runconfig=_request.session['current_runconfig'])
        return simplejson.dumps({})



    else:
        assert False, 'Unhandled aciton: %s'% action
    print 'Action', action
    return simplejson.dumps({})


@dajaxice_register
def overview_update_sim_gui_batch(_request, simfile_ids):
    print 'Updating!'
    ensure_config()

    if isinstance(simfile_ids, int):
        simfile_ids = [simfile_ids]
    elif isinstance(simfile_ids, basestring):
        simfile_ids = [int(tok) for tok in simfile_ids.split()]
    else:
        assert False
    res = []
    for simfile_id in simfile_ids:

        try:
            sim_file = SimFile.get_tracked_sims(id=simfile_id)
        except SimFile.DoesNotExist:
            continue

        if not sim_file.does_file_exist():
            sim_file.delete()
            continue

        runconfig = _request.session['current_runconfig']
        last_run = sim_file.get_last_run(runconfig=runconfig)
        exec_date = last_run.execution_data_string() if last_run is not None else ""
        res.append(  {'sim_id':simfile_id,
                     'state':sim_file.get_status(runconfig=runconfig),
                     'is_queued':sim_file.is_queued(runconfig=runconfig),
                     'latest_exec_id':last_run.id if last_run else "",
                     'latest_exec_date': str(exec_date),
                     'latest_exec_duration': last_run.execution_time if last_run else '' ,
                    } ) 

    v = simplejson.dumps(res)
    print 'Retunrign from AJax'
    #print v
    return v





@dajaxice_register
def overview_update_sim_gui(_request, simfile_id):
    print 'Dajax call recieved', simfile_id
    from views import ensure_config
    ensure_config(_request)
    print 'A'
    sim_file = SimFile.get_tracked_sims(id=simfile_id)
    runconfig = _request.session['current_runconfig']
    print 'B'
    last_run = sim_file.get_last_run(runconfig=runconfig)
    print 'Bb'
    exec_date = last_run.execution_data_string() if last_run is not None else ""
    print 'C'
    v = simplejson.dumps(
                {'sim_id':simfile_id,
                 'state':sim_file.get_status(runconfig=runconfig),
                 'is_queued':sim_file.is_queued(runconfig=runconfig),
                 'latest_exec_id':last_run.id if last_run else "",
                 'latest_exec_date': exec_date,
                } )
    print 'Finsihed'
    return v


@dajaxice_register
def overview_clear_sim_queue(_request):
    SimQueueEntry.objects.all()\
            .filter(status = SimQueueEntryState.Waiting)\
            .delete()
    return simplejson.dumps({})

@dajaxice_register
def overview_delete_simfile(_request, simfile_id):
    ensure_config()

    try:
        sim_file = SimFile.get_tracked_sims(id=simfile_id)
        sim_file.delete()
    except:
        pass
    return simplejson.dumps({})



#!/usr/bin/python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# Copyright (c) 2012 Michael Hull.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------

import os

import mreorg
from mreorg.utils import ScriptUtils
from mreorg.layouts import FigureOptions
from mreorg.config import MReOrgConfig


class FigFormat:

    EPS = 'eps'
    SVG = 'svg'
    PDF = 'pdf'
    PNG = 'png'


class PlotManager:

    fig_num = 0
    figures_saved = []
    figures_saved_nums = []  # This is the numbers
    figures_saved_filenames = []

    _fig_loc = """_output/figures/{modulename}/{figtype}/"""
    _fig_name = """fig{fignum:03d}_{figname}.{figtype}"""
    autosave_default_image_filename_tmpl = _fig_loc + _fig_name

    autosave_image_formats = None
    _all_autosave_image_formats = [FigFormat.EPS, FigFormat.PDF,
                                   FigFormat.PNG, FigFormat.SVG]

    @classmethod
    def save_figure(
        cls,
        figname='',
        figure=None,
        filename_tmpl=None,
        figtypes=None,
        remap_dot_to_underscore=False,
        ):

        if remap_dot_to_underscore:
            figname = figname.replace('.', '-')

        if not filename_tmpl:
            filename_tmpl = cls.autosave_default_image_filename_tmpl
        if not figtypes:
            # Has it been set explicity in the class??
            if cls.autosave_image_formats is not None:
                figtypes = cls.autosave_image_formats

            # Is it specified in a layout config file:
            elif FigureOptions.default_autosave_formats is not None:
                figtypes = FigureOptions.default_autosave_formats
            # Or the rc file?
            elif 'Settings' in MReOrgConfig.config and \
                 'mreorg' in MReOrgConfig.config['Settings'] and \
                 'default_autosave_formats' in MReOrgConfig.config['Settings']['mreorg']:
                figtypes = MReOrgConfig.config['Settings']['mreorg']['default_autosave_formats']

            # No?, then lets default to everything:
            else:
                figtypes = cls._all_autosave_image_formats

        assert isinstance(figtypes, list), 'figtypes: %s <%s>'%(figtypes, type(figtypes))

        # Get the figure:
        import pylab
        fig = (figure if figure else pylab.gcf())

        # Some small changes:
        fig.subplots_adjust(bottom=0.15)

        # Find the module this function was called from:

        modname = ScriptUtils.get_calling_script_file(include_ext=False)

        # Print what we are saving:
        subst_dict = {
            'modulename': modname,
            'fignum': PlotManager.fig_num,
            'figname': figname,
            'figtype': '{%s}' % ','.join(figtypes),
            }
        print 'PlotManger saving: ', filename_tmpl.format(**subst_dict)

        # For each filetype:
        for figtype in figtypes:

            # Create the filename:
            subst_dict = {
                'modulename': modname,
                'fignum': PlotManager.fig_num,
                'figname': figname,
                'figtype': figtype,
                }

            filename = filename_tmpl.format(**subst_dict)
            filename = filename.replace(':', '=')
            assert not ':' in filename, 'For windows compatibility'

            # Save the figure:
            full_filename = os.path.join(os.getcwd(), filename)
            mreorg.ensure_directory_exists(full_filename)
            try:
                fig.savefig(full_filename)
            except ValueError as e:
                print 'mreorg error: unable to save figure: ', full_filename
                print e
                print '(ignoring error)'
            PlotManager.figures_saved.append(fig)
            PlotManager.figures_saved_nums.append(fig.number)
            PlotManager.figures_saved_filenames.append(full_filename)

        # Increment the fignum:
        PlotManager.fig_num = PlotManager.fig_num + 1

    @classmethod
    def save_active_figures(cls):
        import matplotlib

        active_figures = [manager.canvas.figure for manager in
                          matplotlib._pylab_helpers.Gcf.get_all_fig_managers()]
        active_figures_new = [a for a in active_figures if not a.number
                              in cls.figures_saved_nums]

        # Save the new figures:
        for fig in active_figures_new:
            if fig in PlotManager.figures_saved:
                continue

            PlotManager.save_figure(figname='Autosave_figure_%d'
                                    % fig.number, figure=fig)







#!/usr/bin/python
# -*- coding: utf-8 -*-

import re


class LyxFile(object):

    def __init__(self, filename):
        self.filename = filename

        with open(self.filename) as f:
            self.contents = f.read()

    def get_child_files(self):
        res = []
        im_re = re.compile(r'^\s*filename\s+(.*?)$', re.MULTILINE)
        for match in im_re.finditer(self.contents):
            dep_filename = match.groups()[0]
            res.append(dep_filename)

        return res





#!/usr/bin/python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# Copyright (c) 2012 Michael Hull.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------

import os
import mreorg

from mreorg.scriptflags import ScriptFlags
from mreorg.layouts import FigureOptions
import datetime

# Lets monkey-patch matplotlib!
# ===============================

if not ScriptFlags.MREORG_DONTIMPORTMATPLOTLIB:
    import matplotlib
    is_running_headless = not os.environ.get('DISPLAY', None)


    # Preconfiguration, before we import pylab:
    if ScriptFlags.MREORG_MPLCONFIG_FILE:
        print 'Custom MPLConfig', ScriptFlags.MREORG_MPLCONFIG_FILE
        # Load the config file:
        from configobj import ConfigObj
        config = ConfigObj(ScriptFlags.MREORG_MPLCONFIG_FILE)

        # Setup the backend:
        backend = None
        if is_running_headless:
            if 'mpl_backend_headless' in config['options']:
                backend = config['options']['mpl_backend_headless']
            else:
                backend = config['options']['mpl_backend']
        else:
            backend = config['options']['mpl_backend']

        assert backend, 'No backend set for config: %s' % ScriptFlags.MREORG_MPLCONFIG_FILE
        matplotlib.use(backend)

        # Setup the rc-params values:
        mpl_rcparams = config.get('matplotlib', {})

        # Downscale values set in mreorg, if needs be. (This is to work around a bug(?) in matplotlib SVG backend):
        downscale_options = ['font.size','axes.labelsize','legend.fontsize','xtick.labelsize','ytick.labelsize']
        if FigureOptions.downscale_fontsize:
            for k in downscale_options:
                if k in mpl_rcparams.keys():
                    mpl_rcparams[k] = float(mpl_rcparams[k]) / FigureOptions.downscale_fontsize

        for (k, v) in mpl_rcparams.items():
            print 'Setting: %s to %s' % (k, v)
            matplotlib.rcParams[k] = v

    # Monkey-Patch 'matplotlib.show()' and 'pylab.show()', allowing us
    # to disable them, and/or to save figures to disk.
    import pylab
    orig_mplshow = matplotlib.pylab.show


    def show(*args, **kwargs):
        from mreorg.scriptplots import PlotManager
        # Should we save all the figures?
        if ScriptFlags.MREORG_SAVEALL:
            PlotManager.save_active_figures()

        # Should we actually display this:
        if ScriptFlags.MREORG_NOSHOW:
            pass
        else:
            orig_mplshow(*args, **kwargs)


    matplotlib.pylab.show = show
    pylab.show = show


    # Monkey-Patch 'matplotlib.savefig()' and 'pylab.savefig()', allowing us
    # to save to directories that don't exist by automatically creating them:
    orig_mplsavefig = matplotlib.pylab.savefig
    def savefig(filename, *args, **kwargs):
        from mreorg.layouts import FigureOptions
        from mreorg.utils import ScriptUtils
        if ScriptFlags.MREORG_SAVEFIGADDINFO or FigureOptions.is_draft:
            F = pylab.gcf()
            (x, y) = F.get_size_inches()
            txt = 'Size: x=%2.2f y=%2.2f (inches)' % (x, y)
            txt += '\n' + 'On: %s' % datetime.datetime.today().strftime('%d, %h %Y (%H:%M)')
            txt += '\n' + 'Using MPLCONFIGFILE: %s' % ScriptFlags.MREORG_MPLCONFIG_FILE
            txt += '\n' + filename.split('/')[-1]
            txt += '\n' + ScriptUtils.get_calling_script_file(include_ext=True)
            pylab.figtext(0.0, 0.5, txt, backgroundcolor='white')

        if ScriptFlags.MREORG_AUTOMAKEDIRS:
            mreorg.ensure_directory_exists(filename)
        return orig_mplsavefig(filename, *args, transparent=True, **kwargs)

    matplotlib.pylab.savefig = savefig
    pylab.savefig = savefig





    ## Trigger saving all images at the end of the program run:
    #if ScriptFlags.MREORG_SAVEALL:
    #    
    #    from mreorg.atexithandlers import AtExitHandler
    #    def _save_all_at_exit(*args,**kwargs):
    #        print 'Saving atexit:'
    #        from mreorg.scriptplots import PlotManager
    #        PlotManager.save_active_figures()


    #    AtExitHandler.add_handler(_save_all_at_exit)









    # Monkey patch xlabel, ylabel, so the default is
    # multialignment='center'
    from matplotlib import axes
    set_xlabel_old = axes.Axes.set_xlabel
    set_ylabel_old = axes.Axes.set_ylabel

    def set_xlabel_new(self, *args, **kwargs):
        if not 'multialignment' in kwargs:
            kwargs['multialignment'] = 'center'
        return set_xlabel_old(self, *args, **kwargs)


    def set_ylabel_new(self, *args, **kwargs):
        if not 'multialignment' in kwargs:
            kwargs['multialignment'] = 'center'
        return set_ylabel_old(self, *args, **kwargs)

    hack_options = mreorg.MReOrgConfig.config['Settings']['mreorg']['Hacks']
    if hack_options['xlabel_multialign_centre']:
        axes.Axes.set_xlabel = set_xlabel_new
    if hack_options['ylabel_multialign_centre']:
        axes.Axes.set_ylabel = set_ylabel_new


    # SVG hack
    # SVG output is a problem, because of a conversion between pt to px, which I don't really understand.
    # we interecept the call to 'draw_text' at a really low level, in order to rescale the font:
    do_hack=hack_options['svg_downscale'] 
    do_hack=True
    if matplotlib.get_backend() == 'svg' and FigureOptions.downscale_fontsize_hack and do_hack:
        import matplotlib.backends.backend_svg as svg
        def myfunc(self, ctx, x, y,  text, fp, *args,**kwargs):
            fp = fp.copy()
            fp.set_size(fp.get_size() / 1.25) 
            orig = getattr(self, 'mreorg_draw_text')
            return orig( ctx,x,y, text, fp,*args, **kwargs)
        # Rename the original method
        setattr( svg.RendererSVG, 'mreorg_draw_text', getattr(svg.RendererSVG,'draw_text') )
        setattr( svg.RendererSVG, 'draw_text', myfunc)


# Hook in the coverage
if ScriptFlags.MREORG_ENABLECOVERAGE:
    import mreorg.config
    print 'Activating Coverage'
    coverage_opdir = mreorg.config.MReOrgConfig.get_coverage_store_dir()
    os.environ['COVERAGE_PROCESS_START'] = mreorg.config.MReOrgConfig.get_coverage_configfile()

    import coverage
    coverage.process_startup()



if ScriptFlags.MREORG_CURATIONRUN:
    #assert False
    from mreorg.curator.backend_sim.db_writer_hooks import CurationSimDecorator
    CurationSimDecorator.activate()




from __future__ import with_statement
from StringIO import StringIO
from xml.sax import saxutils
from keyword import kwlist as PYTHON_KWORD_LIST

__all__ = ['Builder', 'Element']
__license__ = 'BSD'
__version__ = '0.2.1'
__author__ = "Jonas Galvez <http://jonasgalvez.com.br/>"
__contributors__ = ["bbolli <http://github.com/bbolli/>",
                    "masklinn <http://github.com/masklinn/>"]

class Builder:
    
    def __init__(self, encoding='utf-8', indent=' '*2, version=None):
        self._document = StringIO()
        self._encoding = encoding
        self._indent = indent
        self._indentation = 0
        if version is not None:
            self.write('<?xml version="%s" encoding="%s"?>\n' % (
                version, encoding
            ))
    
    def __getattr__(self, name):
        return Element(name, self)
        
    def __getitem__(self, name):
        return Element(name, self)
    
    def __str__(self):
        return self._document.getvalue().encode(self._encoding).strip()
        
    def __unicode__(self):
        return self._document.getvalue().decode(self._encoding).strip()
        
    def write(self, content):
        """Write raw content to the document"""
        if type(content) is not unicode:
            content = content.decode(self._encoding)
        self._document.write('%s' % content)

    def write_escaped(self, content):
        """Write escaped content to the document"""
        self.write(saxutils.escape(content))
        
    def write_indented(self, content):
        """Write indented content to the document"""
        self.write('%s%s\n' % (self._indent * self._indentation, content))

builder = Builder # 0.1 backward compatibility

class Element:
    
    PYTHON_KWORD_MAP = dict([(k + '_', k) for k in PYTHON_KWORD_LIST])
    
    def __init__(self, name, builder):
        self.name = self._nameprep(name)
        self.builder = builder
        self.attributes = {}
        
    def __enter__(self):
        """Add a parent element to the document"""
        self.builder.write_indented('<%s%s>' % (
            self.name, self._serialized_attrs()
        ))
        self.builder._indentation += 1
        return self
        
    def __exit__(self, type, value, tb):
        """Add close tag to current parent element"""
        self.builder._indentation -= 1
        self.builder.write_indented('</%s>' % self.name)
        
    def __call__(*args, **kargs):
        """Add a child element to the document"""
        self = args[0]
        self.attributes.update(kargs)
        if len(args) > 1:
            value = args[1]
            if value is None:
                self.builder.write_indented('<%s%s />' % (
                    self.name, self._serialized_attrs()
                ))
            else:
                value = saxutils.escape(value)
                self.builder.write_indented('<%s%s>%s</%s>' % (
                    self.name, self._serialized_attrs(), value, self.name
                ))
        return self

    def _serialized_attrs(self):
        """Serialize attributes for element insertion"""
        serialized = []
        for attr, value in self.attributes.items():
            serialized.append(' %s=%s' % (
                self._nameprep(attr), saxutils.quoteattr(value)
            ))
        return ''.join(serialized)

    def _nameprep(self, name):
        """Undo keyword and colon mangling"""
        name = Element.PYTHON_KWORD_MAP.get(name, name)
        return name.replace('__', ':')



#!/usr/bin/python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# Copyright (c) 2012 Michael Hull.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------

import inspect
import os

import hashlib
import tokenize
import token
import contextlib


class ScriptUtils(object):

    @classmethod
    def get_calling_script(cls):
        frame = inspect.stack()

        callee_frame = frame[-1]
        callee_file = callee_frame[1]
        return callee_file

    @classmethod
    def get_calling_script_directory(cls):
        return os.path.dirname(cls.get_calling_script())

    @classmethod
    def get_calling_script_file(cls, include_ext):
        filename = os.path.basename(cls.get_calling_script())
        if include_ext:
            return filename
        else:
            return os.path.splitext(filename)[0]


def extract_docstring_from_fileobj(fileobj):
    for token_data in tokenize.generate_tokens(fileobj.readline):
        tok = token_data[0]
        text = token_data[1]
        if tok in [tokenize.COMMENT, tokenize.NL]:
            continue
        elif tok in [tokenize.NAME, tokenize.ENDMARKER]:
            return None
        elif tok == tokenize.STRING:
            text = text.strip()
            if text.startswith('"""'):
                text = text[3:]
            if text.endswith('"""'):
                text = text[:-3]
            return text.strip()
        else:
            tokenname = token.tok_name[tok]
            msg = 'Unexpected token %s (%s)' % (tokenname, tok)
            raise ValueError(msg)
    return None


def ensure_directory_exists(filename, is_dir=False):
    dirname = os.path.dirname(filename) if not is_dir else filename
    if not os.path.exists(dirname) and dirname.strip():
        os.makedirs(dirname)
    return filename


def get_file_sha1hash(filename):
    hash_obj = hashlib.sha1()
    with open(filename) as fileobj:
        hash_obj.update(fileobj.read())
    return hash_obj.hexdigest()


@contextlib.contextmanager
def switch_into_working_directory(path):
    """A context manager which changes the working directory to the given
    path, and then changes it back to its previous value on exit.
    """
    prev_cwd = os.getcwd()
    os.chdir(path)
    try:
        yield None
    finally:
        os.chdir(prev_cwd)




#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import subprocess
import ast
import re
import sys


# A class that walks over python file contents, and extracts out
# string parameter calls to a function:
# ========================================

class MyFileWalker(ast.NodeVisitor):
    def __init__(self, funcname, argpos, *args, **kwargs):
        super(MyFileWalker, self).__init__(*args,**kwargs)
        self.calls = []
        self.argpos = argpos
        self.funcname = funcname

    def visit_Call(self, node):
        funcname = self.vis_attr(node.func)
        if funcname == self.funcname:
            arg = node.args[self.argpos]

            try:
                self.calls.append(arg.s)
            except:
                pass

    def vis_attr(self, n):
        if isinstance(n, basestring):
            return n
        elif isinstance(n, ast.Name):
            return self.vis_attr(n.id)
        elif isinstance(n, ast.Attribute):
            return self.vis_attr(n.attr)

        # Ignore more complex ast elements: indexing, subcalls
        elif isinstance(n, (ast.Subscript, ast.Call)):
            return None
        else:
            assert False, ' Unexpected node found %s' % type(n)

    @classmethod
    def get_all_func_calls(cls, src_contents, funcname, argpos=0):
        a = ast.parse(src_contents)
        x = cls(funcname=funcname, argpos=argpos)
        x.visit(a)
        return x.calls


# ========================================



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import collections
from functools import partial
import shutil


import xml.etree.ElementTree as ET
import xml
import xml.sax

try:
    import xmlwitch
except ImportError:
    print "Unable to import xmlwitch (needed to write XML/SVG files)"



def normalise_measurement(length):
    # Return size in mm:
    try:
        pxs = float(length)
        length = '%fpt' % (pxs * 0.8)
    except:
        pass

    if length.endswith('pt'):
        return float(length.replace('pt', '')) * 0.035277778 * 10.
    if length.endswith('cm'):
        return float(length.replace('cm', '')) * 10.

    assert False, 'unable to read length: %s' % length


def getWidthHeight(filename):
    root = ET.parse(filename).getroot()
    height = normalise_measurement(root.attrib['height'])
    width = normalise_measurement(root.attrib['width'])

    (height, width) = (int(height), int(width))
    return (width, height)


class MyXMLParser(xml.sax.handler.ContentHandler):

    def get_style_dict(self, attrs):
        toks = [tok.split(':') for tok in attrs.get('style','').split(';') if tok]
        return  dict( toks )

    def update_global_style_dict(self, style_dict, element):
        for (k, v) in style_dict.items():
            self.svg_data.all_styles[element][k].add(v)

    def do_path(self, attrs):
        style_dict = self.get_style_dict(attrs)
        self.update_global_style_dict(style_dict=style_dict, element='path')




    def do_text(self, attrs):
        style_dict = self.get_style_dict(attrs)
        self.update_global_style_dict(style_dict=style_dict, element='text')

    def do_g(self, attrs):
        pass

    def do_marker(self, attrs):
        pass

    def do_tspan(self, attrs):
        pass

    def do_svg(self, attrs):
        pass

    def do_image(self, attrs):
        pass

    def do_rect(self, attrs):
        pass
    def do_circle(self, attrs):
        pass
    def do_ellipse(self, attrs):
        pass
    def do_polygon(self, attrs):
        pass
    def do_polyline(self, attrs):
        pass
    def do_style(self, attrs):
        pass
    def do_line(self, attrs):
        pass


    def startElement(self, name, attrs):
        self.svg_data.n_nodes+=1

        if 'transform' in attrs.keys():
            tr = attrs['transform']
            if tr.startswith('rotate') or tr.startswith('translate'):
                pass
            else:
                self.svg_data.transforms.append(tr)
            #print name, attrs['transform']

        self.parse_stack.append( [] )

        # Sanity check:
        colored_properties = ['fill', 'stroke', 'stop-color', 'flood-color','lighting-color']
        if len( set(attrs.keys()) & set(colored_properties) ) != 0:
            print  'Unexpected properties'
            print 'Node', name, attrs.keys()


        # Ignored elements:
        ignored_elements = ['defs', 'sodipodi:namedview','sodipodi:guide', 'metadata',
                            'rdf:RDF', 'cc:Work', 'inkscape:grid',
                            'dc:format', 'dc:type', 'dc:title',
                            'mask','clipPath','radialGradient','stop','linearGradient','filter', 'feGaussianBlur',
                            'flowRoot', 'flowRegion','flowPara',
                            'use', ]

        method_lut = {
            'text': self.do_text,
            'rect': self.do_rect,
            'circle': self.do_circle,
            'ellipse': self.do_ellipse,
            'g': self.do_g,
            'svg': self.do_svg,
            'text': self.do_text,
            'marker': self.do_marker,
            'tspan': self.do_tspan,
            'style': self.do_style,
            'image': self.do_image,
            'text': self.do_text,
            'path': self.do_path,
            'line': self.do_line,
            'polyline': self.do_polyline,
            'polygon': self.do_polygon,
            }


        if name in method_lut.keys():
            method_lut[name](attrs)
        elif name in ignored_elements:
            pass
        else:
            print 'Unknown node:', name
            assert False



    def endElement(self, name):
        #self.parse_stack.pop()
        pass


    def __init__(self):

        self.svg_data = SVGData()
        self.parse_stack=[]


        #self.transforms=[]




class SVGData(object):
    def __init__(self):
        self.n_nodes = 0

        self.all_styles = collections.defaultdict( partial(collections.defaultdict,set ) )
        self.transforms = []

        # Load some details:
        self.x_mm = 10
        self.y_mm = 10




class FileObj(object):
    def __init__(self, filename):
        self.filename = filename

    @property
    def short_filename(self):
        return os.path.split( self.filename )[-1]
    @property
    def full_filename(self):
        return os.path.join( os.getcwd(), self.filename )

    @property
    def size_MB(self):
        return os.path.getsize(self.filename) / 1.0e6


import pickle

class SVGFile(FileObj):
    xml_parse_cache_filename='.xmlcache'

    @classmethod
    def load_parse_cache(cls,):
        if os.path.exists(cls.xml_parse_cache_filename):
            with open(cls.xml_parse_cache_filename) as f:
                parse_cache = pickle.load(f)
        else:
            parse_cache = {}
        return parse_cache

    @classmethod
    def save_parse_cache(cls,parse_cache):
        with open(cls.xml_parse_cache_filename,'w') as f:
            pickle.dump(parse_cache,f)


    def __init__(self, filename):
        super( SVGFile, self).__init__(filename=filename)


        with open(self.filename) as f:
            self.contents = f.read()

        print filename
        self.svg_data = self.parse_xml()

        print str(self)



    def parse_xml(self):

        parse_cache=self.load_parse_cache()
        if not self.filename in parse_cache:

            my_parser = MyXMLParser()
            xml.sax.parse(self.filename, handler=my_parser)
            parse_cache[self.filename] = my_parser.svg_data
            self.save_parse_cache(parse_cache)

        return parse_cache[self.filename]







    def __str__(self,):

        return '<SVGFile: [%3.2f MB; %4d nodes] %s (%dx%d) NTransforms: %s>' % (
                self.size_MB,
                self.svg_data.n_nodes,
                self.short_filename,
                self.svg_data.x_mm,
                self.svg_data.y_mm,
                len(self.svg_data.transforms) )




    def build_html_details(self, xml=None, ):

        if xml is None:
            xml = xmlwitch.Builder(version='1.0', encoding='utf-8')


        with xml.div():
            with xml.h1():
                with xml.a(name=self.short_filename):
                    #xml.write_escaped(self.filename)
                    xml.write_escaped(self.short_filename)

            with xml.image(src=self.full_filename):
                pass

            #for each element type: g, path, text, etc
            for (element,attrs) in sorted(self.svg_data.all_styles.items()):
                with xml.table():

                    # for each style type (color, fill, etc):
                    for (attr,vals) in sorted(attrs.items()):

                        # Print each values
                        for i,val in enumerate(vals):
                            with xml.tr():
                                xml.td( element if i==0 else '' )
                                xml.td( attr )
                                xml.td(val)

            xml.write_escaped( str(','.join(self.svg_data.transforms) ) )

        return  xml



#def format_

class SVGFileSet(object):
    def __init__(self, svgfiles):
        self.svgfiles = svgfiles


    def build_summary_table(self, xml):

        all_styles = collections.defaultdict( partial(collections.defaultdict, partial(collections.defaultdict,set ) ) )

        for svgfile in self.svgfiles:
            for (element, data) in svgfile.svg_data.all_styles.items():
                for (prop, values) in data.items():
                    for value in values:
                        all_styles[element][prop][value].add(svgfile)


        xml.h1('Summary')
        with xml.table():

            for element in sorted(all_styles):
                for prop in sorted(all_styles[element]):
                    for value in sorted(all_styles[element][prop]):

                        with xml.tr():

                            kwargs = {}
                            if value.startswith('#'):
                                kwargs['bgcolor'] = value

                            xml.td( element  )
                            xml.td( prop  )
                            xml.td( value, **kwargs )
                            svgfiles= sorted(all_styles[element][prop][value])
                            with xml.td():
                                xml.write( ','.join( ['<a href="#%s">%s</a>' % ( svgfile.short_filename, svgfile.short_filename) for svgfile in  svgfiles]) )


                        #for i,svgfile in enumerate(sorted(all_styles[element][prop][value])):
                        #        with xml.tr():

                        #            kwargs = {}
                        #            if value.startswith('#'):
                        #                kwargs['bgcolor'] = value

                        #            xml.td( element if i==0 else '' )
                        #            xml.td( prop if i==0 else '' )
                        #            xml.td( value if i==0 else '', **kwargs )
                        #            xml.td( svgfile.short_filename )



        #['node']['property'] -> ['file1','file2','file3'...]

        from matplotlib.colors import ColorConverter
        #pass
        #colors = set()

        import numpy as np
        import matplotlib.pyplot as plt
        from mpl_toolkits.mplot3d import Axes3D
        #from mpl_toolkits.mplot3d import Axes3D

        fig = plt.figure()
        ax = fig.add_subplot(111, projection='3d')


        for element in sorted(all_styles):
            for prop in sorted(all_styles[element]):
                for value in sorted(all_styles[element][prop]):
                    if value.startswith('#'):
                        files = all_styles[element][prop][value]
                        s = float(len(files)) * 3

                        colors = np.array( ColorConverter().to_rgb(value) ) 
                        colors = colors.reshape( (1,3))
                        print colors
                        print colors.shape
                        l = ax.scatter(colors[:,0], colors[:,1], colors[:,2], c=colors, s=s, picker=5)
                        l.src_color = value
                        l.src_files = files





        def onpick(event):
            print 'On event', event, event.artist
            l = event.artist
            print l.src_color
            print [ f.filename for f in l.src_files]
        fig.canvas.mpl_connect('pick_event', onpick)

        plt.show()



    def build_html_output(self, output_dir='./image_html/'):

        xml = xmlwitch.Builder(version='1.0', encoding='utf-8')


        # Summary Table of all tags and colors:
        self.build_summary_table(xml)

        # Create the output:
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        # General table:
        for svgfile in sorted( self.svgfiles, key=lambda o:o.filename ):
            svgfile.build_html_details(xml=xml)

            # Copy files across:
            shutil.copyfile(svgfile.full_filename, os.path.join(output_dir,svgfile.short_filename ))



        with open(os.path.join(output_dir,'index.html'),'w' ) as f:
            f.write(str(xml))








#!/usr/bin/python
# -*- coding: utf-8 -*-

import os

from mreorg.scriptflags import ScriptFlags

try:
    from validate import Validator
except ImportError:
    print "Unable to import 'Validator'"


class _FigureLayouts(object):

    def __init__(self):

        # Load the config file:
        if ScriptFlags.MREORG_MPLCONFIG_FILE:
            from configobj import ConfigObj
            config = ConfigObj(ScriptFlags.MREORG_MPLCONFIG_FILE,interpolation='template')
            config.interpolation = 'template'

            for k in config.get('layouts', {}):
                self.__dict__[str(k)] = config['layouts'][k]

    def get_xy_in(self, xname, yname):
        print self.__dict__
        x = float(self.__dict__[xname]) / 25.4
        y = float(self.__dict__[yname]) / 25.4
        return (x, y)






class _FigureOptions(object):

    _defaults = {
            'draft':False,
            'default_autosave_formats' : None,
            'downscale_fontsize_hack' :False,
            }

    
    local_dir = os.path.dirname(__file__)
    _configspecfile = os.path.join(local_dir, '../../mplconfigs/mplconfig.spec')

    def __init__(self):
        self.default_autosave_formats = None
        # Load the defaults:
        self.__dict__.update(_FigureOptions._defaults)

        # Load the config file:
        if ScriptFlags.MREORG_MPLCONFIG_FILE:
            from configobj import ConfigObj

            config = ConfigObj(ScriptFlags.MREORG_MPLCONFIG_FILE, interpolation='template', configspec=_FigureOptions._configspecfile)
            validator = Validator()
            result = config.validate(validator)
            assert result == True, 'Invalid Config for mreorg'

            self.__dict__.update(config.get('options',{}))


    @property
    def is_draft(self):
        return self.draft


FigureLayouts = _FigureLayouts()
FigureOptions = _FigureOptions()




#!/usr/bin/python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------

# Copyright (c) 2012 Michael Hull.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------

import os
import fnmatch
import mreorg

import configobj
import validate
from os.path import expanduser
import cStringIO as StringIO

from pkg_resources import resource_string
configspec_str = resource_string(__name__, 'etc/configspec.ini')


class MReOrgConfig(object):

    rcfilename = expanduser('~/.mreorgrc')

    # New:
    # If the config file doesn't exist, make it from the defaults
    # specified in the configspec.ini
    if not os.path.exists(rcfilename):
        config = configobj.ConfigObj(configspec=StringIO.StringIO(configspec_str))
        validator = validate.Validator()
        config.validate(validator,copy=True)
        with open(rcfilename,'w') as f:
            config.write(f)
    config = configobj.ConfigObj(rcfilename, configspec=StringIO.StringIO(configspec_str))
    validator = validate.Validator()
    config.validate(validator)



    @classmethod
    def get_image_store_dir(cls):
        path = cls.config['FileSystem']['Curate']['output_image_dir']
        path = os.path.expanduser(path)
        return mreorg.utils.ensure_directory_exists(path, is_dir=True)

    @classmethod
    def get_coverage_store_dir(cls):
        path = cls.config['FileSystem']['Curate']['coverage_dir']
        path = os.path.expanduser(path)
        print path
        return mreorg.utils.ensure_directory_exists(path, is_dir=True)

    @classmethod
    def get_coverage_configfile(cls):
        path = cls.config['FileSystem']['Curate']['coverage_configfile']
        path = os.path.expanduser(path)
        return mreorg.utils.ensure_directory_exists(path)

    @classmethod
    def get_simulation_sqllite_filename(cls):
        path = cls.config['FileSystem']['Curate']['sqllite_filename']
        path = os.path.expanduser(path)
        return mreorg.utils.ensure_directory_exists(path)


    @classmethod
    def is_non_curated_file(cls, filename):
        patterns = cls.config['Settings']['Curate']['filename_excludes']
        for pattern in patterns:
            if fnmatch.fnmatch(filename, pattern):
                return True
        return False





